@@ -6,6 +6,7 @@ from PIL import Image, DdsImagePlugin
 TEST_FILE_DXT1 = "Tests/images/dxt1-rgb-4bbp-noalpha_MipMaps-1.dds"
 TEST_FILE_DXT3 = "Tests/images/dxt3-argb-8bbp-explicitalpha_MipMaps-1.dds"
 TEST_FILE_DXT5 = "Tests/images/dxt5-argb-8bbp-interpolatedalpha_MipMaps-1.dds"
+TEST_FILE_DX10_BC7 = "Tests/images/bc7-argb-8bpp_MipMaps-1.dds"
 
 
 class TestFileDds(PillowTestCase):
@@ -22,7 +23,6 @@ class TestFileDds(PillowTestCase):
         self.assertEqual(im.mode, "RGBA")
         self.assertEqual(im.size, (256, 256))
 
-        # This target image is from the test set of images, and is exact.
         self.assert_image_equal(target.convert('RGBA'), im)
 
     def test_sanity_dxt5(self):
@@ -37,18 +37,35 @@ class TestFileDds(PillowTestCase):
         self.assertEqual(im.mode, "RGBA")
         self.assertEqual(im.size, (256, 256))
 
-        # Imagemagick, which generated this target image from the .dds
-        # has a slightly different decoder than is standard. It looks
-        # a little brighter. The 0,0 pixel is (00,6c,f8,ff) by our code,
-        # and by the target image for the DXT1, and the imagemagick .png
-        # is giving (00, 6d, ff, ff).  So, assert similar, pretty tight
-        # I'm currently seeing about a 3 for the epsilon.
-        self.assert_image_similar(target, im, 5)
+        self.assert_image_equal(target, im)
 
     def test_sanity_dxt3(self):
-        """Check DXT3 images are not supported"""
-        self.assertRaises(NotImplementedError,
-                          lambda: Image.open(TEST_FILE_DXT3))
+        """Check DXT3 images can be opened"""
+
+        target = Image.open(TEST_FILE_DXT3.replace('.dds', '.png'))
+
+        im = Image.open(TEST_FILE_DXT3)
+        im.load()
+
+        self.assertEqual(im.format, "DDS")
+        self.assertEqual(im.mode, "RGBA")
+        self.assertEqual(im.size, (256, 256))
+
+        self.assert_image_equal(target, im)
+
+    def test_dx10_bc7(self):
+        """Check DX10 images can be opened"""
+
+        target = Image.open(TEST_FILE_DX10_BC7.replace('.dds', '.png'))
+
+        im = Image.open(TEST_FILE_DX10_BC7)
+        im.load()
+
+        self.assertEqual(im.format, "DDS")
+        self.assertEqual(im.mode, "RGBA")
+        self.assertEqual(im.size, (256, 256))
+
+        self.assert_image_equal(target, im)
 
     def test__validate_true(self):
         """Check valid prefix"""
@@ -89,11 +106,10 @@ class TestFileDds(PillowTestCase):
             img_file = f.read()
 
         def short_file():
-            Image.open(BytesIO(img_file[:-100]))
+            im = Image.open(BytesIO(img_file[:-100]))
+            im.load()
 
         self.assertRaises(IOError, short_file)
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
